import 'package:flutter/material.dart';

import 'device/device_descriptor.dart';

class DevicePopupMenuButton extends StatelessWidget {
  const DevicePopupMenuButton({
    Key? key,
    required this.devices,
    required this.currentIndex,
    required this.backgroundColor,
    required this.onDeviceSelected,
    required this.iconData,
    required this.iconColor,
    this.tooltip,
  }) : super(key: key);

  final List<DeviceDescriptor> devices;
  final int currentIndex;
  final Color backgroundColor;
  final ValueSetter<DeviceDescriptor> onDeviceSelected;
  final IconData iconData;
  final Color iconColor;
  final String? tooltip;

  @override
  Widget build(BuildContext context) {
    return Material(
      color: backgroundColor,
      child: PopupMenuButton<DeviceDescriptor>(
        key: key,
        tooltip: tooltip,
        // color: Colors.green,
        padding: EdgeInsets.zero,
        initialValue: devices.first,
        icon: Icon(
          iconData,
          color: iconColor,
        ),
        onSelected: onDeviceSelected,
        itemBuilder: (context) => _getMenuItems(context),
      ),
    );
  }

  List<PopupMenuItem<DeviceDescriptor>> _getMenuItems(BuildContext context) {
    var index = 1;
    final style = TextStyle(color: index == currentIndex ? Colors.white : null);

    return devices.map((e) {
      index++;
      final leading = e.tablet == true ? Icons.tablet : Icons.phone;

      return PopupMenuItem(
        child: ListTile(
          // tileColor: index == currentIndex ? Colors.grey.shade800 : null,
          contentPadding: const EdgeInsets.symmetric(horizontal: 10),
          dense: true,
          title: Text(
            e.name,
            style: style,
          ),
          leading: Icon(leading),
        ),
        value: e,
        enabled: index != currentIndex,
      );
    }).toList();
  }
}
