import 'dart:async';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class StatusBar extends StatefulWidget {
  final double height;
  final double horizontalPadding;
  final Color backgroundColor;
  final bool showBatterieStatusPercentage;
  final String? text;

  const StatusBar({
    Key? key,
    required this.height,
    this.horizontalPadding = 8.0,
    required this.backgroundColor,
    this.showBatterieStatusPercentage = true,
    this.text,
  }) : super(key: key);

  @override
  State<StatusBar> createState() => _StatusBarState();
}

class _StatusBarState extends State<StatusBar> {
  static const padding = 2.0;
  static final color = Colors.white38;

  static final style = TextStyle(
    fontFamily: 'Roboto',
    fontWeight: FontWeight.w500,
    fontSize: 15.0,
    color: color,
  );

  Timer? timer;
  final formatter = DateFormat('HH:mm');
  late final Widget? _text;
  late final Widget? _percentage;
  late String currentTime;

  @override
  void initState() {
    super.initState();
    _text = widget.text == null ? null : Text(widget.text!, style: style.copyWith(fontSize: 12));
    _percentage = widget.showBatterieStatusPercentage ? Text('98%', style: style.copyWith(fontSize: 12)) : null;
    currentTime = formatter.format(DateTime.now());
    timer = Timer.periodic(const Duration(seconds: 1), (timer) {
      setState(() {
        if (mounted) {
          currentTime = formatter.format(DateTime.now());
        }
      });
    });
  }

  @override
  void dispose() {
    super.dispose();
    timer?.cancel();
  }

  @override
  Widget build(BuildContext context) {
    double iconSize = max((widget.height - padding * 2) - 2, 16.0);

    return Container(
      padding: EdgeInsets.symmetric(horizontal: widget.horizontalPadding),
      height: widget.height,
      color: widget.backgroundColor,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Center(
            child: Text(
              currentTime,
              style: style,
            ),
          ),
          const SizedBox(width: 8),
          if (_text != null) Center(child: _text),
          Expanded(
            child: Container(),
          ),
          if (iconSize > 0)
            Padding(
              padding: const EdgeInsets.fromLTRB(padding, padding, 2 * padding, padding),
              child: Icon(
                Icons.network_wifi,
                color: color,
                size: iconSize,
              ),
            ),
          if (_percentage != null) Center(child: _percentage),
          if (iconSize > 0)
            Padding(
              padding: const EdgeInsets.fromLTRB(0, padding, 0, padding),
              child: Icon(
                Icons.battery_charging_full,
                color: color,
                size: iconSize,
              ),
            ),
        ],
      ),
    );
  }
}
