// import 'package:collection/collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../device/device_descriptor.dart';
import 'menu_position.dart';

class SimulatorSettings {
  /// Enables or disables the DeviceSimulator
  final bool enable;

  /// The [brightness] decides how to draw the status bar (black or white).
  final Brightness brightness;

  /// The color of the iOS multitasking bar that is available on newer
  /// iOS devices without a home button.
  final Color iOSMultitaskBarColor;

  final bool showNavigationBar;

  final Color statusBarBackgroundColor;

  final Color navigationBarBackgroundColor;

  final Orientation orientation;

  final Color backgroundColor;

  final int initialDeviceIndex;

  final TargetPlatform initialPlatform;

  final bool showBottomBar;

  final bool showDeviceSlider;

  final Color accentColor;

  final Color dividerColor;

  final bool showColorSelector;

  final List<DeviceDescriptor> iosDevices;

  final List<DeviceDescriptor> androidDevices;

  final MenuPosition menuPosition;

  SimulatorSettings({
    Color? backgroundColor,
    Color? accentColor,
    Color? dividerColor,
    this.enable = true,
    this.brightness = Brightness.dark,
    this.iOSMultitaskBarColor = Colors.grey,
    this.showNavigationBar = false,
    this.statusBarBackgroundColor = Colors.black26,
    this.navigationBarBackgroundColor = Colors.black26,
    this.orientation = Orientation.portrait,
    this.initialDeviceIndex = 0,
    this.initialPlatform = TargetPlatform.android,
    this.showBottomBar = true,
    this.showDeviceSlider = true,
    this.showColorSelector = false,
    required this.iosDevices,
    required this.androidDevices,
    this.menuPosition = MenuPosition.left,
  })  : this.backgroundColor = backgroundColor ?? Colors.grey.shade900,
        this.accentColor = accentColor ?? Colors.lightBlueAccent.shade100,
        this.dividerColor = dividerColor ?? accentColor ?? Colors.grey.shade700;

  // SimulatorAppSettings copyWith({
  //   bool? enable,
  //   Brightness? brightness,
  //   Color? iOSMultitaskBarColor,
  //   bool? showNavigationBar,
  //   Color? statusBarBackgroundColor,
  //   Color? navigationBarBackgroundColor,
  //   Orientation? orientation,
  //   Color? backgroundColor,
  //   int? initialDeviceIndex,
  //   TargetPlatform? initialPlatform,
  //   bool? showBottomBar,
  //   bool? showDeviceSlider,
  //   Color? accentColor,
  //   Color? dividerColor,
  //   bool? showColorSelector,
  //   List<DeviceDescriptor>? iosDevices,
  //   List<DeviceDescriptor>? androidDevices,
  // }) {
  //   return SimulatorAppSettings(
  //     enable: enable ?? this.enable,
  //     brightness: brightness ?? this.brightness,
  //     iOSMultitaskBarColor: iOSMultitaskBarColor ?? this.iOSMultitaskBarColor,
  //     showNavigationBar: showNavigationBar ?? this.showNavigationBar,
  //     statusBarBackgroundColor: statusBarBackgroundColor ?? this.statusBarBackgroundColor,
  //     navigationBarBackgroundColor: navigationBarBackgroundColor ?? this.navigationBarBackgroundColor,
  //     orientation: orientation ?? this.orientation,
  //     backgroundColor: backgroundColor ?? this.backgroundColor,
  //     initialDeviceIndex: initialDeviceIndex ?? this.initialDeviceIndex,
  //     initialPlatform: initialPlatform ?? this.initialPlatform,
  //     showBottomBar: showBottomBar ?? this.showBottomBar,
  //     showDeviceSlider: showDeviceSlider ?? this.showDeviceSlider,
  //     accentColor: accentColor ?? this.accentColor,
  //     dividerColor: dividerColor ?? this.dividerColor,
  //     showColorSelector: showColorSelector ?? this.showColorSelector,
  //     iosDevices: iosDevices ?? this.iosDevices,
  //     androidDevices: androidDevices ?? this.androidDevices,
  //   );
  // }

  Map<String, dynamic> toMap() {
    return {
      'enable': enable,
      'brightness': describeEnum(brightness),
      'iOSMultitaskBarColor': iOSMultitaskBarColor.value,
      'showNavigationBar': showNavigationBar,
      'statusBarBackgroundColor': statusBarBackgroundColor.value,
      'navigationBarBackgroundColor': navigationBarBackgroundColor.value,
      'orientation': describeEnum(orientation),
      'backgroundColor': backgroundColor.value,
      'initialDeviceIndex': initialDeviceIndex,
      'initialPlatform': describeEnum(initialPlatform),
      'showBottomBar': showBottomBar,
      'showDeviceSlider': showDeviceSlider,
      'accentColor': accentColor.value,
      'dividerColor': dividerColor.value,
      'showColorSelector': showColorSelector,
      // 'iosDevices': iosDevices?.map((x) => x.toMap())?.toList(),
      // 'androidDevices': androidDevices?.map((x) => x.toMap())?.toList(),
    };
  }

  // factory SimulatorAppSettings.fromMap(Map<String, dynamic> map) {
  //   return SimulatorAppSettings(
  //     enable: map['enable'],
  //     brightness: BrightnessExtension.fromMap(map['brightness']),
  //     iOSMultitaskBarColor: Color(map['iOSMultitaskBarColor']),
  //     showNavigationBar: map['showNavigationBar'],
  //     statusBarBackgroundColor: Color(map['statusBarBackgroundColor']),
  //     navigationBarBackgroundColor: Color(map['navigationBarBackgroundColor']),
  //     orientation: OrientationExtension.fromMap(map['orientation']),
  //     backgroundColor: Color(map['backgroundColor']),
  //     initialDeviceIndex: map['initialDeviceIndex'],
  //     initialPlatform: TargetPlatformExtension.fromMap(map['initialPlatform']),
  //     showBottomBar: map['showBottomBar'],
  //     showDeviceSlider: map['showDeviceSlider'],
  //     accentColor: Color(map['accentColor']),
  //     dividerColor: Color(map['dividerColor']),
  //     showColorSelector: map['showColorSelector'],
  //     // iosDevices: List<DeviceDescriptor>.from(map['iosDevices']?.map((x) => DeviceDescriptor.fromMap(x))),
  //     // androidDevices: List<DeviceDescriptor>.from(map['androidDevices']?.map((x) => DeviceDescriptor.fromMap(x))),
  //   );
  // }

  // Map<String, dynamic> toMap() {
  //   return {
  //     'enable': enable,
  //     'brightness': describeEnum(brightness),
  //     'iOSMultitaskBarColor': iOSMultitaskBarColor.value,
  //     'showNavigationBar': showNavigationBar,
  //     'statusBarBackgroundColor': statusBarBackgroundColor.value,
  //     'navigationBarBackgroundColor': navigationBarBackgroundColor.value,
  //     'orientation': describeEnum(orientation),
  //     'backgroundColor': backgroundColor.value,
  //     'initialDeviceIndex': initialDeviceIndex,
  //     'initialPlatform': describeEnum(initialPlatform),
  //     'showBottomBar': showBottomBar,
  //     'showDeviceSlider': showDeviceSlider,
  //     'accentColor': accentColor.value,
  //     'dividerColor': dividerColor.value,
  //     'showColorSelector': showColorSelector,
  //   };
  // }

  // factory SimulatorAppSettings.fromMap(Map<String, dynamic> map) {
  //   return SimulatorAppSettings(
  //     enable: map['enable'],
  //     brightness: toEnum<Brightness>(Brightness.values, 'brightness', map), //  Brightness.fromMap(map['brightness']),
  //     iOSMultitaskBarColor: Color(map['iOSMultitaskBarColor']),
  //     showNavigationBar: map['showNavigationBar'],
  //     statusBarBackgroundColor: Color(map['statusBarBackgroundColor']),
  //     navigationBarBackgroundColor: Color(map['navigationBarBackgroundColor']),
  //     orientation: toEnum<Orientation>(Orientation.values, 'orientation', map),
  //     backgroundColor: Color(map['backgroundColor']),
  //     initialDeviceIndex: map['initialDeviceIndex'],
  //     initialPlatform: toEnum<TargetPlatform>(TargetPlatform.values, 'initialPlatform', map),
  //     showBottomBar: map['showBottomBar'],
  //     showDeviceSlider: map['showDeviceSlider'],
  //     accentColor: Color(map['accentColor']),
  //     dividerColor: Color(map['dividerColor']),
  //     showColorSelector: map['showColorSelector'],
  //   );
  // }

  // String toJson() => json.encode(toMap());

  // factory SimulatorAppSettings.fromJson(String source) => SimulatorAppSettings.fromMap(json.decode(source));

  @override
  String toString() {
    return 'SimulatorAppSettings(enable: $enable, brightness: $brightness, iOSMultitaskBarColor: $iOSMultitaskBarColor, showNavigationBar: $showNavigationBar, statusBarBackgroundColor: $statusBarBackgroundColor, navigationBarBackgroundColor: $navigationBarBackgroundColor, orientation: $orientation, backgroundColor: $backgroundColor, initialDeviceIndex: $initialDeviceIndex, initialPlatform: $initialPlatform, showBottomBar: $showBottomBar, showDeviceSlider: $showDeviceSlider, accentColor: $accentColor, dividerColor: $dividerColor, showColorSelector: $showColorSelector, iosDevices: $iosDevices, androidDevices: $androidDevices)';
  }

  // @override
  // bool operator ==(Object other) {
  //   if (identical(this, other)) return true;
  //   final listEquals = const DeepCollectionEquality().equals;

  //   return other is SimulatorSettings &&
  //       other.enable == enable &&
  //       other.brightness == brightness &&
  //       other.iOSMultitaskBarColor == iOSMultitaskBarColor &&
  //       other.showNavigationBar == showNavigationBar &&
  //       other.statusBarBackgroundColor == statusBarBackgroundColor &&
  //       other.navigationBarBackgroundColor == navigationBarBackgroundColor &&
  //       other.orientation == orientation &&
  //       other.backgroundColor == backgroundColor &&
  //       other.initialDeviceIndex == initialDeviceIndex &&
  //       other.initialPlatform == initialPlatform &&
  //       other.showBottomBar == showBottomBar &&
  //       other.showDeviceSlider == showDeviceSlider &&
  //       other.accentColor == accentColor &&
  //       other.dividerColor == dividerColor &&
  //       other.showColorSelector == showColorSelector &&
  //       listEquals(other.iosDevices, iosDevices) &&
  //       listEquals(other.androidDevices, androidDevices);
  // }

  @override
  int get hashCode {
    return enable.hashCode ^
        brightness.hashCode ^
        iOSMultitaskBarColor.hashCode ^
        showNavigationBar.hashCode ^
        statusBarBackgroundColor.hashCode ^
        navigationBarBackgroundColor.hashCode ^
        orientation.hashCode ^
        backgroundColor.hashCode ^
        initialDeviceIndex.hashCode ^
        initialPlatform.hashCode ^
        showBottomBar.hashCode ^
        showDeviceSlider.hashCode ^
        accentColor.hashCode ^
        dividerColor.hashCode ^
        showColorSelector.hashCode ^
        iosDevices.hashCode ^
        androidDevices.hashCode;
  }
}
