import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../constants.dart';
import '../display/display_container.dart';
import '../settings/menu_position.dart';
import 'simulator_controller.dart';
import '../menu/menu_bar.dart';
import '../menu/menu_bar_vertical.dart';

class SimulatorContainer extends StatelessWidget {
  const SimulatorContainer({
    Key? key,
    // required this.device,
    required this.builder,
  }) : super(key: key);

  // final DeviceDescriptor device;
  final WidgetBuilder builder;

  @override
  Widget build(BuildContext context) {
    final mq = MediaQuery.of(context).size;

    return Consumer<SimulatorController>(builder: (context, controller, _) {
      final device = controller.currentDevice;
      final screenSize = controller.orientation == Orientation.portrait ? device.size : device.size.flipped;

      final pos = controller.menuPosition;

      if (pos == MenuPosition.top || pos == MenuPosition.bottom) {
        final displayContainer = SizedBox(
          height: mq.height,
          child: DisplayContainer(
            screenSize: screenSize,
            child: builder,
            backgroundColor: controller.settings.backgroundColor,
            navigationBarBackgroundColor: controller.settings.navigationBarBackgroundColor,
            showBatterieStatusPercentage: device.platform != TargetPlatform.iOS,
            showAndroidNavigationBar: controller.settings.showNavigationBar,
          ),
        );
        final menu = Container(
          // height: 72,
          child: Material(
            child: MenuBar(controller: controller),
          ),
        );

//        final children = pos == MenuPosition.top ? [hiddenMenu, displayContainer] : [displayContainer, hiddenMenu];
        Widget hiddenMenu = Visibility(child: menu, visible: controller.showMenuBar);
        if (pos == MenuPosition.bottom) {
          hiddenMenu = Padding(padding: EdgeInsets.only(top: mq.height - 72), child: hiddenMenu);
        }
        return Stack(
//          mainAxisAlignment: MainAxisAlignment.center,
          // crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [displayContainer, hiddenMenu],
        );
        // return Column(
        //   key: ValueKey(device),
        //   mainAxisAlignment: MainAxisAlignment.spaceBetween,
        //   mainAxisSize: MainAxisSize.max,
        //   children: children,
        // );
      } else {
        final displayContainer = SizedBox(
          height: mq.height,
          width: mq.width,
          child: DisplayContainer(
            screenSize: screenSize,
            child: builder,
            backgroundColor: controller.settings.backgroundColor,
            navigationBarBackgroundColor: controller.settings.navigationBarBackgroundColor,
            showBatterieStatusPercentage: device.platform != TargetPlatform.iOS,
            showAndroidNavigationBar: controller.settings.showNavigationBar,
          ),
        );

        Widget menuBar = Visibility(
          child: Material(
            child: MenuBarVertical(controller: controller),
          ),
          visible: controller.showMenuBar,
        );

        if (pos == MenuPosition.right) {
          menuBar = Padding(padding: EdgeInsets.only(left: mq.width - 72), child: menuBar);
        }

        return Stack(
          children: [
            Material(
              color: controller.settings.backgroundColor,
              child: InkWell(
                onLongPress: () => controller.toggleMenu(),
                onDoubleTap: () => controller.toggleMenu(),
                child: Container(
                  width: double.infinity,
                  height: double.infinity,
                  color: Colors.transparent, // controller.settings.backgroundColor,
                  child: const Align(
                    alignment: Alignment.topCenter,
                    child: Text('DFFP3 | dotup IT solutions'),
                  ),
                ),
              ),
            ),
            menuBar,
            if (controller.overlayWidget != null && controller.showMenuBar)
              Selector<SimulatorController, bool>(
                selector: (context, controller) {
                  return controller.shouldUpdateOverlay();
                },
                builder: (context, value, child) {
                  return controller.overlayWidget!(context);
                },
              ),
            displayContainer,
          ],
        );
      }
    });
  }
}
